@@ -1,30 +1,36 @@
+#![allow(clippy::incompatible_msrv)]
+
 use bencher::{benchmark_group, benchmark_main, Bencher};
 use once_cell::sync::Lazy;
 use rcgen::{
-    date_time_ymd, BasicConstraints, Certificate, CertificateParams, CertificateRevocationList,
-    CertificateRevocationListParams, IsCa, KeyIdMethod, KeyUsagePurpose, RevocationReason,
-    RevokedCertParams, SerialNumber, PKCS_ECDSA_P256_SHA256,
+    date_time_ymd, BasicConstraints, CertificateParams, CertificateRevocationListParams,
+    CertifiedKey, IsCa, KeyIdMethod, KeyPair, KeyUsagePurpose, RevocationReason, RevokedCertParams,
+    SerialNumber, PKCS_ECDSA_P256_SHA256,
 };
 
+use pki_types::CertificateRevocationListDer;
 use std::fs::File;
 use std::hint::black_box;
 use std::io::{ErrorKind, Read, Write};
 use std::path::Path;
 use std::sync::Mutex;
 
-use webpki::{BorrowedCertRevocationList, CertRevocationList};
+use webpki::{BorrowedCertRevocationList, CertRevocationList, OwnedCertRevocationList};
 
 /// Lazy initialized CRL issuer to be used when generating CRL data. Includes
 /// `KeyUsagePurpose::CrlSign` key usage bit.
-static CRL_ISSUER: Lazy<Mutex<Certificate>> = Lazy::new(|| {
-    let mut issuer_params = CertificateParams::new(vec!["crl.issuer.example.com".to_string()]);
+static CRL_ISSUER: Lazy<Mutex<CertifiedKey>> = Lazy::new(|| {
+    let mut issuer_params =
+        CertificateParams::new(vec!["crl.issuer.example.com".to_string()]).unwrap();
     issuer_params.is_ca = IsCa::Ca(BasicConstraints::Unconstrained);
     issuer_params.key_usages = vec![
         KeyUsagePurpose::KeyCertSign,
         KeyUsagePurpose::DigitalSignature,
         KeyUsagePurpose::CrlSign,
     ];
-    Mutex::new(Certificate::from_params(issuer_params).unwrap())
+    let key_pair = KeyPair::generate_for(&PKCS_ECDSA_P256_SHA256).unwrap();
+    let cert = issuer_params.self_signed(&key_pair).unwrap();
+    Mutex::new(CertifiedKey { cert, key_pair })
 });
 
 /// Number of revoked certificates to include in the small benchmark CRL. Produces a CRL roughly
@@ -44,12 +50,15 @@ const FAKE_SERIAL: &[u8] = &[0xC0, 0xFF, 0xEE];
 /// Try to load a DER bytes from `crl_path`. If that file path does not exist, generate a CRL
 /// with `revoked_count` revoked certificates, write the DER encoding to `crl_path` and return the
 /// newly created DER bytes.
-fn load_or_generate(crl_path: impl AsRef<Path> + Copy, revoked_count: usize) -> Vec<u8> {
+fn load_or_generate(
+    crl_path: impl AsRef<Path> + Copy,
+    revoked_count: usize,
+) -> CertificateRevocationListDer<'static> {
     match File::open(crl_path) {
         Ok(mut crl_file) => {
             let mut crl_der = Vec::new();
             crl_file.read_to_end(&mut crl_der).unwrap();
-            crl_der
+            crl_der.into()
         }
         Err(e) => match e.kind() {
             ErrorKind::NotFound => match File::create(crl_path) {
@@ -68,7 +77,7 @@ fn load_or_generate(crl_path: impl AsRef<Path> + Copy, revoked_count: usize) ->
 }
 
 /// Create a new benchmark CRL with `revoked_count` revoked certificates.
-fn generate_crl(revoked_count: usize) -> Vec<u8> {
+fn generate_crl(revoked_count: usize) -> CertificateRevocationListDer<'static> {
     let mut revoked_certs = Vec::with_capacity(revoked_count);
     (0..revoked_count).for_each(|i| {
         revoked_certs.push(RevokedCertParams {
@@ -83,13 +92,14 @@ fn generate_crl(revoked_count: usize) -> Vec<u8> {
         this_update: date_time_ymd(2023, 6, 17),
         next_update: date_time_ymd(2024, 6, 17),
         crl_number: SerialNumber::from(1234),
-        alg: &PKCS_ECDSA_P256_SHA256,
         key_identifier_method: KeyIdMethod::Sha256,
+        issuing_distribution_point: None,
         revoked_certs,
     };
-    let crl = CertificateRevocationList::from_params(crl).unwrap();
     let issuer = CRL_ISSUER.lock().unwrap();
-    crl.serialize_der_with_signer(&issuer).unwrap()
+    crl.signed_by(&issuer.cert, &issuer.key_pair)
+        .unwrap()
+        .into()
 }
 
 /// Benchmark parsing a small CRL file into a borrowed representation.
@@ -103,12 +113,7 @@ fn bench_parse_borrowed_crl_small(c: &mut Bencher) {
 fn bench_parse_owned_crl_small(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/small.crl.der", SMALL_CRL_CERT_COUNT);
 
-    c.iter(|| {
-        BorrowedCertRevocationList::from_der(&crl_bytes)
-            .unwrap()
-            .to_owned()
-            .unwrap()
-    });
+    c.iter(|| OwnedCertRevocationList::from_der(&crl_bytes).unwrap());
 }
 
 /// Benchmark parsing a medium CRL file into a borrowed representation..
@@ -122,12 +127,7 @@ fn bench_parse_borrowed_crl_medium(c: &mut Bencher) {
 fn bench_parse_owned_crl_medium(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/medium.crl.der", MEDIUM_CRL_CERT_COUNT);
 
-    c.iter(|| {
-        BorrowedCertRevocationList::from_der(&crl_bytes)
-            .unwrap()
-            .to_owned()
-            .unwrap()
-    });
+    c.iter(|| OwnedCertRevocationList::from_der(&crl_bytes).unwrap());
 }
 
 /// Benchmark parsing a large CRL file into a borrowed representation..
@@ -141,19 +141,16 @@ fn bench_parse_borrowed_crl_large(c: &mut Bencher) {
 fn bench_parse_owned_crl_large(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/large.crl.der", LARGE_CRL_CERT_COUNT);
 
-    c.iter(|| {
-        BorrowedCertRevocationList::from_der(&crl_bytes)
-            .unwrap()
-            .to_owned()
-            .unwrap()
-    });
+    c.iter(|| BorrowedCertRevocationList::from_der(&crl_bytes).unwrap());
 }
 
 /// Benchmark searching a small CRL file in borrowed representation for a serial that does not
 /// appear. Doesn't include the time it takes to parse the CRL in the benchmark task.
 fn bench_search_borrowed_crl_small(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/small.crl.der", SMALL_CRL_CERT_COUNT);
-    let crl = BorrowedCertRevocationList::from_der(&crl_bytes).unwrap();
+    let crl: CertRevocationList = BorrowedCertRevocationList::from_der(&crl_bytes)
+        .unwrap()
+        .into();
 
     c.iter(|| black_box(assert!(matches!(crl.find_serial(FAKE_SERIAL), Ok(None)))));
 }
@@ -162,10 +159,9 @@ fn bench_search_borrowed_crl_small(c: &mut Bencher) {
 /// appear. Doesn't include the time it takes to parse the CRL in the benchmark task.
 fn bench_search_owned_crl_small(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/small.crl.der", SMALL_CRL_CERT_COUNT);
-    let crl = BorrowedCertRevocationList::from_der(&crl_bytes)
+    let crl: CertRevocationList = OwnedCertRevocationList::from_der(&crl_bytes)
         .unwrap()
-        .to_owned()
-        .unwrap();
+        .into();
 
     c.iter(|| black_box(assert!(matches!(crl.find_serial(FAKE_SERIAL), Ok(None)))));
 }
@@ -174,7 +170,9 @@ fn bench_search_owned_crl_small(c: &mut Bencher) {
 /// appear. Doesn't include the time it takes to parse the CRL in the benchmark task.
 fn bench_search_borrowed_crl_medium(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/medium.crl.der", MEDIUM_CRL_CERT_COUNT);
-    let crl = BorrowedCertRevocationList::from_der(&crl_bytes).unwrap();
+    let crl: CertRevocationList = BorrowedCertRevocationList::from_der(&crl_bytes)
+        .unwrap()
+        .into();
 
     c.iter(|| black_box(assert!(matches!(crl.find_serial(FAKE_SERIAL), Ok(None)))));
 }
@@ -183,10 +181,9 @@ fn bench_search_borrowed_crl_medium(c: &mut Bencher) {
 /// appear. Doesn't include the time it takes to parse the CRL in the benchmark task.
 fn bench_search_owned_crl_medium(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/medium.crl.der", MEDIUM_CRL_CERT_COUNT);
-    let crl = BorrowedCertRevocationList::from_der(&crl_bytes)
+    let crl: CertRevocationList = OwnedCertRevocationList::from_der(&crl_bytes)
         .unwrap()
-        .to_owned()
-        .unwrap();
+        .into();
 
     c.iter(|| black_box(assert!(matches!(crl.find_serial(FAKE_SERIAL), Ok(None)))));
 }
@@ -195,7 +192,9 @@ fn bench_search_owned_crl_medium(c: &mut Bencher) {
 /// appear. Doesn't include the time it takes to parse the CRL in the benchmark task.
 fn bench_search_borrowed_crl_large(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/large.crl.der", LARGE_CRL_CERT_COUNT);
-    let crl = BorrowedCertRevocationList::from_der(&crl_bytes).unwrap();
+    let crl: CertRevocationList = BorrowedCertRevocationList::from_der(&crl_bytes)
+        .unwrap()
+        .into();
 
     c.iter(|| black_box(assert!(matches!(crl.find_serial(FAKE_SERIAL), Ok(None)))));
 }
@@ -204,10 +203,9 @@ fn bench_search_borrowed_crl_large(c: &mut Bencher) {
 /// appear. Doesn't include the time it takes to parse the CRL in the benchmark task.
 fn bench_search_owned_crl_large(c: &mut Bencher) {
     let crl_bytes = load_or_generate("./benches/large.crl.der", LARGE_CRL_CERT_COUNT);
-    let crl = BorrowedCertRevocationList::from_der(&crl_bytes)
+    let crl: CertRevocationList = OwnedCertRevocationList::from_der(&crl_bytes)
         .unwrap()
-        .to_owned()
-        .unwrap();
+        .into();
 
     c.iter(|| black_box(assert!(matches!(crl.find_serial(FAKE_SERIAL), Ok(None)))));
 }
